'use strict';

const http = require('http');
const path = require('path');
const watch = require('./watch');

const DEFAULT_PORT = 8080;
const RE_NODE_MODULES = /node_modules/;

const originalCreateServer = http.createServer;
let ownModules;

/**
 * Create app server
 * @param {string} mainpath
 * @param {{ port: number, reloadServer: object }} [options]
 * @param {() => void} [fn]
 * @returns {Promise<{ destroy: () => void }>}
 */
module.exports = async function appServer(
  mainpath,
  { port = DEFAULT_PORT, reloadServer } = {},
  fn
) {
  if (process.env.PORT === undefined) {
    process.env.PORT = port;
  }
  if (ownModules === undefined) {
    ownModules = Object.keys(require.cache);
  }

  return await factory(path.resolve(mainpath), reloadServer, fn);
};

/**
 * Start app server
 * @param {string} main
 * @returns {Promise<[string]>}
 */
function start(main) {
  return new Promise((resolve, reject) => {
    http.createServer = (...args) => {
      const server = originalCreateServer(...args);

      server.on('error', reject);
      server.on('listening', () => resolve([server, getAppModules()]));

      http.createServer = originalCreateServer;

      return server;
    };

    try {
      require(main);
    } catch (err) {
      reject(err);
    }
  });
}

/**
 * Stop running server
 * @param {http.Server} server
 * @returns {Promise<void>}
 */
function stop(server) {
  return new Promise((resolve) => {
    server.close((err) => {
      if (err) {
        // ignore
      }
      resolve();
    });
  });
}

/**
 * Clear app modules from module cache
 * @param {[string]} appModules
 */
function clearAppModules(appModules) {
  for (const m of appModules) {
    require.cache[m] = undefined;
  }
}

/**
 * Retrieve app modules (excluding node_modules)
 * @returns {[string]}
 */
function getAppModules() {
  return Object.keys(require.cache).filter(
    (m) => !ownModules.includes(m) && !RE_NODE_MODULES.test(m)
  );
}

/**
 * Factory for return instance
 * @param {string} main
 * @param {object} reloadServer
 * @param {() => void} [fn]
 * @returns {{ destroy: () => void }}
 */
async function factory(main, reloadServer, fn) {
  let [server, appModules] = await start(main);
  const watcher = await watch(reloadServer ? process.cwd() : appModules, async (filepath) => {
    clearAppModules(appModules);

    try {
      await stop(server);

      const [newServer, newAppModules] = await start(main);

      for (const m of newAppModules) {
        if (!appModules.includes(m)) {
          watcher.add(m);
        }
      }

      server = newServer;
      appModules = newAppModules;
    } catch (err) {
      console.error(err);
    }

    if (reloadServer) {
      reloadServer.refresh(filepath);
    }

    fn && fn();
  });

  return {
    destroy() {
      clearAppModules(appModules);
      appModules = [];
      watcher.close();
      return stop(server);
    }
  };
}
